﻿//
//	SmartRedFox.com
//
//--------------------------------------
//The Totaliser (v0.5) - 24/08/2010
//--------------------------------------

(function($) {
  	var defaults = {
        digits: 9,
        currency: true,
        currencySign: "£"
    };
	var events = $({});
	var settings;

  $.fn.totaliser = function(action, new_options) {
    if (typeof(action) == 'object' || action == null) {
      new_options = action;
      action = 'initialize';
    }
    setOptions(new_options);
    
    if (action == 'initialize')
	{
		//Set up the totaliser
		this.each(function () {
            var $this = $(this);
            //Check to make sure it's a textbox
            if ($this.is(':text')) {
                //START
				//Html to use for items
				var figureHtml = '<div class="figure"><ul class="zero"><li class="displayed">0</li><li>1</li><li>2</li><li>3</li><li>4</li><li>5</li><li>6</li><li>7</li><li>8</li><li>9</li><li>0</li></ul></div>';
				var seperatorHtml = '<div class="seperator">,</div>';				
				
                var totaliserValue = $this.val();

                //Create the initial html
                var totaliserHtml = '';
                var index = 1;
                for (i = settings.digits; i > 0; i--) {
                    totaliserHtml = figureHtml + totaliserHtml;
                    //Check to see if we need to enter a comma
                    if (index % 3 == 0 && i != 1) {
                        totaliserHtml = seperatorHtml + totaliserHtml
                    }
                    index++;
                }

				//Add the currency symbol in if needed
				var currencyText = "";
				if(settings.currency){currencyText = '<div id="currency"><ul><li>'+ settings.currencySign +'</li></ul></div>'}
				totaliserHtml = '<div id="totaliser" class="clearfix rounded innerShadow"><div id="counter"><div id="sign"><ul><li>+</li></ul></div>' + currencyText + totaliserHtml + "</div></div>";

                $this.addClass("TotaliserValue");
                
                //Hide the input field . We've got an exciting totaliser instead now!
                $this.hide();

                //add the totaliser where the input field was
                $this.after(totaliserHtml);
            }
        });
	}
	else
	{
		updateTotaliser(action);
	}
    return settings;
  };

	function updateTotaliser(newValue){

	    //Update the text box
	    $(".TotaliserValue").val(newValue).attr("Value", newValue);

	    //DOM objects
	    var counterBox = $('#totaliser'); //totaliser container
	    var figures = $('.figure ul', counterBox); //Figure lists
	    var sign = $('li', '#sign');    
	    
	    //If negative and we're not already showing the sign
	    if(newValue < 0 && sign.text() != '-'){
	        sign.fadeOut(1000,function(){$(this).text('-')}).fadeIn(1000);
			//Needs jquery ui to work
	        counterBox.animate({ color: "#82000a"}, 1000);
	    }
	    else{
	        if(newValue > 0 && sign.text() != '+')
	        {
	            sign.fadeOut(1000).text("+").fadeIn(1000);
				//Needs jquery ui to work
	            counterBox.animate({ color: "#23412a"}, 1000);
	        }
	    }

		//Get rid of negative sign for animation
		if (newValue < 0) {newValue *= -1 }
		
	    //var digitHeight = $('.figure:first', counterBox).Height();
	    var digitHeight = 60;

	    //Animation speed (Milliseconds per digit)
		var digitMoveSpeed = 300;	    

		//Pad value to needed number of chars and remove decimal places
	    var paddedValue = $.strPad(roundNumber(newValue, 0), figures.length);

		//Split the value into individual figures
		var newCharArray = new Array();
	    newCharArray = (paddedValue.toString()).split('');
		
		//Used for zero shading
	    var allZeros = true;
		
		figures.each(
					function (index) {
						
						var currentFigure = $(this);
						
						//Get the displayed item (for reuse)
						var currentItem = $('li.displayed',currentFigure).removeClass('displayed');					
						
						//Get selected value and target value
						var currentVal =  parseInt(currentItem.text());
						var targetVal = parseInt(newCharArray[index]);				
						
						//zero shading
						if(targetVal == 0 && allZeros){				
							currentFigure.addClass('zero');						
						}
						else
						{
							currentFigure.removeClass('zero');
							allZeros = false;
						}
						
						//If we don't need to do anything return
						if(currentVal == targetVal){
							currentItem.addClass('displayed');
							return;
						}										
						
						//Simple first: move without having to loop
						if(currentVal < targetVal){

							difference = targetVal - currentVal;						
							currentFigure.stop(true,true).animate({ top:((digitHeight * currentVal) + (digitHeight * difference)) * -1}, difference * digitMoveSpeed, 'linear');
							
							//Set the new selected item
							$('li:eq('+ targetVal +")",currentFigure).addClass('displayed');
							return;
						}		
						else{
							//More complex. Loop back to start when we hit the beginning
							if(currentVal > targetVal){								
							
								//Move to top 0																		
								currentFigure.stop(true,true).animate({ top:(digitHeight * 10) * -1}, (11 - currentVal) * digitMoveSpeed, 'linear',
									function(){
										currentFigure.css({top:0}).animate({top:-1 * targetVal * digitHeight},(targetVal + 1) * digitMoveSpeed,'linear');
									}
								);
								
								//Set the new selected item							
								$('li:eq('+ targetVal +")",currentFigure).addClass('displayed');
							}
							return;
						}	
						
					}
		);	
	}
  
  function setOptions(new_options) {
    settings = $.extend({}, defaults, settings, new_options);
  }
  
  return this;
})(jQuery);


//String padding function
$.strPad = function (i, l, s) {
    var o = i.toString();
    if (!s) { s = '0'; }
    while (o.length < l) {
        o = s + o;
    }
    return o;
};

//Number rounding function
function roundNumber(num, dec) {
	var result = Math.round(num * Math.pow(10, dec)) / Math.pow(10, dec);
	return result;
}